@@ -467,7 +467,7 @@ BOOL cmd_stackframe(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         char* obj_kind[T_TYPE_MAX] = {
             NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor", "job", "object", "function", "class", "external program", "completion", "external object"
         };
-        int size = snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
+        int size = snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[STYPE(object2)]);
 
         if(!fd_write(nextout, buf, size)) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
@@ -492,7 +492,7 @@ BOOL cmd_funinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sObject* running_object = runinfo->mRunningObject;
 
-    if(running_object && (TYPE(running_object) == T_FUN || TYPE(running_object) == T_CLASS) && gRunInfoOfRunningObject)
+    if(running_object && (STYPE(running_object) == T_FUN || STYPE(running_object) == T_CLASS) && gRunInfoOfRunningObject)
     {
         sRunInfo* runinfo2 = gRunInfoOfRunningObject;
 
@@ -723,8 +723,7 @@ BOOL cmd_rehash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
-
-sObject* gPrompt;
+sObject* gPrompt = NULL;
 
 BOOL cmd_prompt(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
@@ -737,7 +736,20 @@ BOOL cmd_prompt(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
-BOOL cmd_block(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+sObject* gRPrompt = NULL;
+
+BOOL cmd_rprompt(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    if(runinfo->mBlocksNum == 1) {
+        gRPrompt = block_clone_on_gc(runinfo->mBlocks[0], T_BLOCK, FALSE);
+        uobject_put(gXyzshObject, "_rprompt", gRPrompt);
+        runinfo->mRCode = 0;
+    }
+
+    return TRUE;
+}
+
+BOOL cmd_ablock(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sObject* fun = runinfo->mRunningObject;
 
