@@ -14,8 +14,7 @@
 #include <signal.h>
 #include <limits.h>
 #include <dirent.h>
-#include <readline/readline.h>
-#include <readline/history.h>
+#include <editline/readline.h>
 
 #if defined(HAVE_CURSES_H)
 #include <curses.h>
@@ -32,10 +31,10 @@ static void str_cut(enum eKanjiCode code, char* mbs, int termsize, char* dest_mb
         wchar_t* wcs_tmp;
         int i;
 
-        wcs = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1)*MB_CUR_MAX);
-        wcs_tmp = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1)*MB_CUR_MAX);
-        if(mbstowcs(wcs, mbs, (termsize+1)*MB_CUR_MAX) == -1) {
-            mbstowcs(wcs, "?????", (termsize+1)*MB_CUR_MAX);
+        wcs = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1));
+        wcs_tmp = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1));
+        if(mbstowcs(wcs, mbs, termsize+1) == -1) {
+            mbstowcs(wcs, "?????", termsize+1);
         }
 
         for(i=0; i<wcslen(wcs); i++) {
@@ -74,6 +73,112 @@ static void str_cut(enum eKanjiCode code, char* mbs, int termsize, char* dest_mb
     }
 }
 
+static void selector_view(BOOL p_view, int scrolltop, int cursor, sObject* nextin, int* markfiles, enum eKanjiCode code, int* n)
+{
+    const int maxx = mgetmaxx();
+    const int maxy = mgetmaxy();
+
+    clear();
+    *n = scrolltop;
+    const int start_line = *n;
+    const int end_line = *n + maxy-1;
+    int y = 0;
+    while(y < maxy-1 && *n < vector_count(SFD(nextin).mLines)) {
+        int attrs = 0;
+        if(*n == cursor) {
+            attrs = A_REVERSE;
+        }
+        if(attrs) attron(attrs);
+
+        char* line = vector_item(SFD(nextin).mLines, *n);
+
+        char head[2];
+        if(markfiles[*n]) {
+            xstrncpy(head, "*", 2);
+        } else {
+            xstrncpy(head, " ", 2);
+        }
+
+        if(str_termlen(code, line) + 1 <= maxx) {
+            if(p_view) {
+                mvprintw(y, 0, "%s", line);
+            } else {
+                mvprintw(y, 0, head);
+                printw("%s", line);
+            }
+            y++;
+        }
+        else {
+            char* p = line;
+
+            char one_line[BUFSIZ];
+            str_cut(code, p, maxx-1, one_line, BUFSIZ);
+            if(p_view) {
+                mvprintw(y, 0, "%s", one_line);
+            }
+            else {
+                mvprintw(y, 0, head);
+                printw("%s", one_line);
+            }
+            y++;
+
+            p+=strlen(one_line);
+
+            while(y < maxy -1 && p < line + strlen(line)) {
+                str_cut(code, p, maxx, one_line, BUFSIZ);
+                mvprintw(y, 0, "%s", one_line);
+                y++;
+
+                p+=strlen(one_line);
+            }
+        }
+
+        if(attrs) attroff(attrs);
+
+        (*n)++;
+    }
+
+    mvprintw(maxy-1, 0, "lines (%d-%d of %d) cursor index (%d)", start_line, end_line, vector_count(SFD(nextin).mLines), cursor);
+}
+
+static void selector_search_forward(sObject* nextin, char* search_str, int* cursor)
+{
+    int i;
+    for(i=*cursor+1; i<vector_count(SFD(nextin).mLines); i++) {
+        char* line = vector_item(SFD(nextin).mLines, i);
+        if(strstr(line, search_str)) {
+            *cursor = i;
+            return;
+        }
+    }
+    for(i=0; i<*cursor; i++) {
+        char* line = vector_item(SFD(nextin).mLines, i);
+        if(strstr(line, search_str)) {
+            *cursor = i;
+            return;
+        }
+    }
+}
+
+static void selector_search_backward(sObject* nextin, char* search_str, int* cursor)
+{
+    int i;
+    for(i=*cursor-1; i>=0; i--) {
+        char* line = vector_item(SFD(nextin).mLines, i);
+        if(strstr(line, search_str)) {
+            *cursor = i;
+            return;
+        }
+    }
+    for(i=vector_count(SFD(nextin).mLines)-1; i>*cursor; i--) {
+        char* line = vector_item(SFD(nextin).mLines, i);
+        if(strstr(line, search_str)) {
+            *cursor = i;
+            return;
+        }
+    }
+}
+
 BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     if(isatty(0) == 0 || isatty(1) == 0) {
@@ -118,8 +223,8 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         code = kEucjp;
     }
 
-
     BOOL multiple = sRunInfo_option(runinfo, "-multiple");
+    sObject* search_str = STRING_NEW_STACK("");
 
     if(runinfo->mFilter) {
         fd_split(nextin, lf, TRUE, FALSE, FALSE);
@@ -171,62 +276,44 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             while(1) {
                 /// view ///
-                clear();
-                int n = scrolltop;
-                int y = 0;
-                while(y < maxy && n < vector_count(SFD(nextin).mLines)) {
-                    int attrs = 0;
-                    if(n == cursor) {
-                        attrs = A_REVERSE;
-                    }
-                    else if(markfiles[n]) {
-                        attrs = A_BOLD;
-                    }
-                    if(attrs) attron(attrs);
-
-                    char* line = vector_item(SFD(nextin).mLines, n);
-
-                    if(str_termlen(code, line) <= maxx) {
-                        mvprintw(y, 0, "%s", line);
-                        y++;
-                    }
-                    else {
-                        char* p = line;
-                        while(p < line + strlen(line)) {
-                            char one_line[BUFSIZ];
-                            str_cut(code, p, maxx, one_line, BUFSIZ);
-                            mvprintw(y, 0, "%s", one_line);
-                            y++;
-
-                            p+=strlen(one_line);
-                        }
-                    }
-
-                    if(attrs) attroff(attrs);
-
-                    n++;
-                }
+                int n;
+                selector_view(FALSE, scrolltop, cursor, nextin, markfiles, code, &n);
                 refresh();
 
                 /// input ///
                 int key = getch();
 
-                if(key == 14 || key == KEY_DOWN) {
+                if(key == 14 || key == KEY_DOWN || key == 'e' || key == 5 || key == 'j' || key == 'n'-'a'+1) {
                     cursor++;
                 }
-                else if(key == 16 || key == KEY_UP) {
+                else if(key == 'y' || key == 'y'-'a'+1 || key == 'k' || key == 'k'-'a'+1 || key == 16 || key == KEY_UP) {
                     cursor--;
                 }
-                else if(key == 4 || key == KEY_NPAGE) {
+                else if(key == 'd' || key == 4) {
                     cursor += (maxy / 2);
                 }
-                else if(key == 21 || key == KEY_PPAGE) {
+                else if(key == 'u' || key == 21) {
                     cursor -= (maxy /2);
                 }
+                else if(key == 'f' || key == 'f'-'a'+1 || key == 'v'-'a'+1 || key == KEY_NPAGE) {
+                    cursor += maxy -1;
+                }
+                else if(key == 27) {
+                    key = getch();
+                    if(key == 'v') {
+                        cursor -= (maxy-1);
+                    }
+                }
+                else if(key == 'b' || key == 'b'-'a'+1 || key == KEY_PPAGE) {
+                    cursor -= (maxy -1);
+                }
                 else if(key == 12) {
                     clear();
                     refresh();
                 }
+                else if(key == 'g') {
+                    cursor = 0;
+                }
                 else if(key == 'q' || key == 3 || key == 7) {
                     err_msg("canceled", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
                     endwin();
@@ -241,6 +328,51 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         markfiles[i] = !markfiles[i];
                     }
                 }
+                else if(key == '/') {
+                    string_put(search_str, "");
+
+                    while(1) {
+                        int dummy;
+                        selector_view(FALSE, scrolltop, cursor, nextin, markfiles, code, &dummy);
+                        const int maxy = mgetmaxy();
+                        mclear_online(maxy-1);
+                        mvprintw(maxy-1, 0, "/%s", string_c_str(search_str));
+                        refresh();
+
+                        int key = getch();
+                        if(key == 10 || key == 13) {
+                            break;
+                        }
+                        else if(key == 8 || key == 127) {
+                            string_trunc(search_str, string_length(search_str)-1);
+                        }
+                        else if(key >= ' ' && key <= '~' || key >= 128) {
+                            string_push_back2(search_str, (char)key);
+                        }
+                    }
+
+                    selector_search_forward(nextin, string_c_str(search_str), &cursor);
+                }
+                else if(key == 'n') {
+                    int dummy;
+                    selector_view(FALSE, scrolltop, cursor, nextin, markfiles, code, &dummy);
+                    const int maxy = mgetmaxy();
+                    mclear_online(maxy-1);
+                    mvprintw(maxy-1, 0, "/%s", string_c_str(search_str));
+                    refresh();
+
+                    selector_search_forward(nextin, string_c_str(search_str), &cursor);
+                }
+                else if(key == 'N') {
+                    int dummy;
+                    selector_view(FALSE, scrolltop, cursor, nextin, markfiles, code, &dummy);
+                    const int maxy = mgetmaxy();
+                    mclear_online(maxy-1);
+                    mvprintw(maxy-1, 0, "/%s", string_c_str(search_str));
+                    refresh();
+
+                    selector_search_backward(nextin, string_c_str(search_str), &cursor);
+                }
                 else if(key == ' ' && multiple) {
                     markfiles[cursor] = !markfiles[cursor];
                     cursor++;
@@ -304,7 +436,7 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 if(cursor < scrolltop) {
                     int i = cursor;
                     int width_sum = 0;
-                    while(width_sum < maxy) {
+                    while(width_sum < (maxy-1)) {
                         char* line = vector_item(SFD(nextin).mLines, i);
                         int width = str_termlen(code, line) / maxx + 1;
                         width_sum += width;
@@ -413,6 +545,7 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         msave_ttysettings();
         msave_screen();
+        mreset_tty();
         initscr();
         raw();
         keypad(stdscr, TRUE);
@@ -455,64 +588,98 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             scrolltop = i +2;
         }
 
+        sObject* search_str = STRING_NEW_STACK("");
+
         while(1) {
             /// view ///
             clear();
-            int n = scrolltop;
-            int y = 0;
-            while(y < maxy && n < vector_count(SFD(nextin2).mLines)) {
-                int attrs = 0;
-                if(n == cursor) {
-                    attrs = A_REVERSE;
-                }
-                else if(markfiles[n]) {
-                    attrs = A_BOLD;
-                }
-                if(attrs) attron(attrs);
-
-                char* line = vector_item(SFD(nextin2).mLines, n);
-
-                if(str_termlen(code, line) <= maxx) {
-                    mvprintw(y, 0, "%s", line);
-                    y++;
-                }
-                else {
-                    char* p = line;
-                    while(p < line + strlen(line)) {
-                        char one_line[BUFSIZ];
-                        str_cut(code, p, maxx, one_line, BUFSIZ);
-                        mvprintw(y, 0, "%s", one_line);
-                        y++;
-
-                        p+=strlen(one_line);
-                    }
-                }
-
-                if(attrs) attroff(attrs);
-
-                n++;
-            }
+            int n;
+            selector_view(TRUE, scrolltop, cursor, nextin2, markfiles, code, &n);
             refresh();
 
             /// input ///
             int key = getch();
 
-            if(key == 14 || key == KEY_DOWN) {
+            if(key == 14 || key == KEY_DOWN || key == 'e' || key == 5 || key == 'j' || key == 'n'-'a'+1) {
                 cursor++;
             }
-            else if(key == 16 || key == KEY_UP) {
+            else if(key == 'y' || key == 'y'-'a'+1 || key == 'k' || key == 'k'-'a'+1 || key == 16 || key == KEY_UP) {
                 cursor--;
             }
-            else if(key == 4 || key == KEY_NPAGE) {
+            else if(key == 'd' || key == 4) {
                 cursor += (maxy / 2);
             }
-            else if(key == 21 || key == KEY_PPAGE) {
+            else if(key == 'u' || key == 21) {
                 cursor -= (maxy /2);
             }
+            else if(key == 'f' || key == 'f'-'a'+1 || key == 'v'-'a'+1 || key == KEY_NPAGE) {
+                cursor += maxy -1;
+            }
+            else if(key == 27) {
+                key = getch();
+                if(key == 'v') {
+                    cursor -= (maxy-1);
+                }
+            }
+            else if(key == 'b' || key == 'b'-'a'+1 || key == KEY_PPAGE) {
+                cursor -= (maxy -1);
+            }
             else if(key == 12) {
                 clear();
                 refresh();
             }
+            else if(key == 'g') {
+                cursor = 0;
+            }
+            else if(key == 12) {
+                clear();
+                refresh();
+            }
+            else if(key == '/') {
+                string_put(search_str, "");
+
+                while(1) {
+                    int dummy;
+                    selector_view(TRUE, scrolltop, cursor, nextin2, markfiles, code, &dummy);
+                    const int maxy = mgetmaxy();
+                    mclear_online(maxy-1);
+                    mvprintw(maxy-1, 0, "/%s", string_c_str(search_str));
+                    refresh();
+
+                    int key = getch();
+                    if(key == 10 || key == 13) {
+                        break;
+                    }
+                    else if(key == 8 || key == 127) {
+                        string_trunc(search_str, string_length(search_str)-1);
+                    }
+                    else if(key >= ' ' && key <= '~' || key >= 128) {
+                        string_push_back2(search_str, (char)key);
+                    }
+                }
+
+                selector_search_forward(nextin2, string_c_str(search_str), &cursor);
+            }
+            else if(key == 'n') {
+                int dummy;
+                selector_view(TRUE, scrolltop, cursor, nextin2, markfiles, code, &dummy);
+                const int maxy = mgetmaxy();
+                mclear_online(maxy-1);
+                mvprintw(maxy-1, 0, "/%s", string_c_str(search_str));
+                refresh();
+
+                selector_search_forward(nextin2, string_c_str(search_str), &cursor);
+            }
+            else if(key == 'N') {
+                int dummy;
+                selector_view(TRUE, scrolltop, cursor, nextin2, markfiles, code, &dummy);
+                const int maxy = mgetmaxy();
+                mclear_online(maxy-1);
+                mvprintw(maxy-1, 0, "/%s", string_c_str(search_str));
+                refresh();
+
+                selector_search_backward(nextin2, string_c_str(search_str), &cursor);
+            }
             else if(key == 'q' || key == 3 || key == 7) {
                 err_msg("p: canceled", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
                 endwin();
@@ -550,7 +717,7 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             if(cursor < scrolltop) {
                 int i = cursor;
                 int width_sum = 0;
-                while(width_sum < maxy) {
+                while(width_sum < (maxy-1)) {
                     char* line = vector_item(SFD(nextin2).mLines, i);
                     int width = str_termlen(code, line) / maxx + 1;
                     width_sum += width;
@@ -600,38 +767,37 @@ BOOL cmd_readline(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     }
 
     if(runinfo->mBlocksNum == 1) {
-        readline_completion_user_castamized(runinfo->mBlocks[0]);
-
-        char* buf = readline(prompt);
+        char* buf = ALLOC editline_user_castamized(prompt, NULL, NULL, -1, runinfo->mBlocks[0]);
 
         if(buf) {
             if(!fd_write(nextout, buf, strlen(buf))) {
                 err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                readline_completion();
+                FREE(buf);
                 return FALSE;
             }
+            FREE(buf);
         }
-
-        readline_completion();
     }
     else {
+        char* buf;
         if(sRunInfo_option(runinfo, "-no-completion")) {
-            readline_no_completion();
+            buf = ALLOC editline_no_completion(prompt, NULL, NULL, -1);
+        }
+        else {
+            buf = ALLOC editline(prompt, NULL, NULL, -1);
         }
-
-        char* buf = readline(prompt);
 
         if(buf) {
             if(!fd_write(nextout, buf, strlen(buf))) {
                 err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                readline_completion();
+                FREE(buf);
                 return FALSE;
             }
+
+            FREE(buf);
         }
-        
-        readline_completion();
     }
 
     runinfo->mRCode = 0;
@@ -741,13 +907,11 @@ static void str_cut2(enum eKanjiCode code, char* mbs, int termsize, char* dest_m
         int i;
         int n;
 
-        wchar_t* wcs 
-            = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1)*MB_CUR_MAX);
-        wchar_t* tmp 
-            = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1)*MB_CUR_MAX);
+        wchar_t* wcs = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1));
+        wchar_t* tmp = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1));
 
-        if(mbstowcs(wcs, mbs, (termsize+1)*MB_CUR_MAX) == -1) {
-            mbstowcs(wcs, "?????", (termsize+1)*MB_CUR_MAX);
+        if(mbstowcs(wcs, mbs, termsize+1) == -1) {
+            mbstowcs(wcs, "?????", termsize+1);
         }
 
         i = 0;
