@@ -30,7 +30,7 @@ BOOL cmd_plusplus(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         sObject* current = runinfo->mCurrentObject;
         sObject* var = access_object(runinfo->mArgsRuntime[1], &current, runinfo->mRunningObject);
 
-        if(var && TYPE(var) == T_STRING) {
+        if(var && STYPE(var) == T_STRING) {
             int num = atoi(string_c_str(var));
             num++;
             char buf[128];
@@ -54,7 +54,7 @@ BOOL cmd_minusminus(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         sObject* current = runinfo->mCurrentObject;
         sObject* var = access_object(runinfo->mArgsRuntime[1], &current, runinfo->mRunningObject);
 
-        if(var && TYPE(var) == T_STRING) {
+        if(var && STYPE(var) == T_STRING) {
             int num = atoi(string_c_str(var));
             num--;
             char buf[128];
