@@ -36,7 +36,7 @@ BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         char*obj_kind[T_TYPE_MAX] = {
             NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor2", "job", "object", "function", "class", "external program", "completion", "external object"
         };
-        int size = snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[TYPE(object2)]);
+        int size = snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[STYPE(object2)]);
 
         if(!fd_write(nextout, buf, size)) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
@@ -76,7 +76,7 @@ BOOL object_info(sObject* object, sObject* nextin, sObject* nextout, sRunInfo* r
 {
     char buf[128];
 
-    switch(TYPE(object)) {
+    switch(STYPE(object)) {
         case T_STRING :
             snprintf(buf, 128, "Type: string\nLength: %d\nMallocLen: %d\n", SSTRING(object).mLen, SSTRING(object).mMallocLen);
 
@@ -272,7 +272,7 @@ BOOL cmd_mrun(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         sObject* current = runinfo->mCurrentObject;
         sObject* klass = access_object(runinfo->mArgsRuntime[1], &current, runinfo->mRunningObject);
 
-        if(klass && TYPE(klass) == T_CLASS) {
+        if(klass && STYPE(klass) == T_CLASS) {
             runinfo->mCurrentObject = runinfo->mRecieverObject;
             if(!run_function(klass, nextin, nextout, runinfo, runinfo->mArgsRuntime + 2, runinfo->mArgsNumRuntime -2, runinfo->mBlocks, runinfo->mBlocksNum)) {
                 return FALSE;
@@ -350,7 +350,7 @@ BOOL cmd_co(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             return FALSE;
         }
 
-        if(new_current && TYPE(new_current) == T_UOBJECT) {
+        if(new_current && STYPE(new_current) == T_UOBJECT) {
             gCurrentObject = new_current;
             runinfo->mRCode = 0;
         }
@@ -367,12 +367,12 @@ BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sObject* running_object = runinfo->mRunningObject;
     if(runinfo->mRunningObject) {
-        switch(TYPE(running_object)) {
+        switch(STYPE(running_object)) {
         case T_FUN:  {
             sObject* parent = SFUN(running_object).mParent;
 
             if(parent) {
-                if(TYPE(parent) == T_NFUN) {
+                if(STYPE(parent) == T_NFUN) {
                     if(gAppType == kATConsoleApp)
                     {
                         if(tcsetpgrp(0, getpgid(0)) < 0) {
@@ -441,8 +441,8 @@ static BOOL entry_fun(sObject* nextin, sObject* nextout, sRunInfo* runinfo, sObj
     if(sRunInfo_option(runinfo, "-inherit")) {
         sObject* parent = uobject_item(object, name);
         if(parent) {
-            if(type == T_FUN && (TYPE(parent) == T_FUN || TYPE(parent) == T_NFUN) 
-                || type == T_CLASS && TYPE(parent) == T_CLASS) 
+            if(type == T_FUN && (STYPE(parent) == T_FUN || STYPE(parent) == T_NFUN) 
+                || type == T_CLASS && STYPE(parent) == T_CLASS) 
             {
                 SFUN(block).mParent = parent;
             }
@@ -562,7 +562,7 @@ BOOL cmd_def(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 return FALSE;
             }
 
-            if(fun && TYPE(fun) == T_FUN) {
+            if(fun && STYPE(fun) == T_FUN) {
                 char* source = SBLOCK(SFUN(fun).mBlock).mSource;
                 if(!fd_write(nextout, source, strlen(source))) {
                     err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
@@ -642,7 +642,7 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 return FALSE;
             }
 
-            if(klass && TYPE(klass) == T_CLASS) {
+            if(klass && STYPE(klass) == T_CLASS) {
                 char* source = SBLOCK(SCLASS(klass).mBlock).mSource;
                 if(!fd_write(nextout, source, strlen(source))) {
                     err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
@@ -699,7 +699,7 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).mLines, 0), TRUE);
                 string_chomp(new_var);
 
-                if(!add_object_to_objects_in_pipe(new_var, runinfo, runinfo))
+                if(!add_object_to_objects_in_pipe(new_var, runinfo, runinfo->mCommand))
                 {
                     return FALSE;
                 }
@@ -720,7 +720,7 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             else {
                 sObject* new_var = STRING_NEW_GC("", TRUE);
 
-                if(!add_object_to_objects_in_pipe(new_var, runinfo, runinfo))
+                if(!add_object_to_objects_in_pipe(new_var, runinfo, runinfo->mCommand))
                 {
                     return FALSE;
                 }
@@ -826,7 +826,7 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     return FALSE;
                 }
 
-                if(var && TYPE(var) == T_STRING) {
+                if(var && STYPE(var) == T_STRING) {
                     if(!fd_write(nextout, string_c_str(var), string_length(var))) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
                         runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
@@ -882,7 +882,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     return FALSE;
                 }
 
-                if(ary && TYPE(ary) == T_VECTOR) {
+                if(ary && STYPE(ary) == T_VECTOR) {
                     char buf[128];
                     int size = snprintf(buf, 128, "%d\n", vector_count(ary));
 
@@ -914,7 +914,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     return FALSE;
                 }
 
-                if(ary && TYPE(ary) == T_VECTOR) {
+                if(ary && STYPE(ary) == T_VECTOR) {
                     if(vector_count(ary) > 0) {
                         int num = atoi(argument);
                         if(num < 0) {
@@ -954,7 +954,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     return FALSE;
                 }
 
-                if(ary && TYPE(ary) == T_VECTOR) {
+                if(ary && STYPE(ary) == T_VECTOR) {
                     int j;
                     for(j=0; j<vector_count(ary); j++) {
                         sObject* var = vector_item(ary, j);
@@ -990,7 +990,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int max = vector_count(SFD(nextin).mLines);
             sObject* new_ary = VECTOR_NEW_GC(16, TRUE);
 
-            if(!add_object_to_objects_in_pipe(new_ary, runinfo, runinfo))
+            if(!add_object_to_objects_in_pipe(new_ary, runinfo, runinfo->mCommand))
             {
                 return FALSE;
             }
@@ -1027,7 +1027,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 sObject* ary;
                 if(sRunInfo_option(runinfo, "-local")) {
                     ary = uobject_item(SFUN(runinfo->mRunningObject).mLocalObjects, runinfo->mArgsRuntime[1]);
-                    if(ary == NULL || TYPE(ary) != T_VECTOR) {
+                    if(ary == NULL || STYPE(ary) != T_VECTOR) {
                         uobject_put(SFUN(runinfo->mRunningObject).mLocalObjects, runinfo->mArgsRuntime[1], ary = VECTOR_NEW_GC(16, TRUE));
                     }
                 }
@@ -1039,7 +1039,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     }
 
                     ary = uobject_item(object, string_c_str(name));
-                    if(ary == NULL || TYPE(ary) != T_VECTOR) {
+                    if(ary == NULL || STYPE(ary) != T_VECTOR) {
                         if(object == runinfo->mCurrentObject) {
                             uobject_put(runinfo->mCurrentObject, runinfo->mArgsRuntime[1], ary = VECTOR_NEW_GC(16, TRUE));
                         }
@@ -1154,7 +1154,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     return FALSE;
                 }
 
-                if(hash && TYPE(hash) == T_HASH) {
+                if(hash && STYPE(hash) == T_HASH) {
                     char buf[128];
                     int size = snprintf(buf, 128, "%d\n", hash_count(hash));
 
@@ -1186,7 +1186,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     return FALSE;
                 }
 
-                if(hash && TYPE(hash) == T_HASH) {
+                if(hash && STYPE(hash) == T_HASH) {
                     sObject* var = hash_item(hash, argument);
 
                     if(var) {
@@ -1219,7 +1219,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 if(!get_object_from_argument(&hash, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
                     return FALSE;
                 }
-                if(hash && TYPE(hash) == T_HASH) {
+                if(hash && STYPE(hash) == T_HASH) {
                     hash_it* it = hash_loop_begin(hash);
                     while(it) {
                         char* key = hash_loop_key(it);
@@ -1265,7 +1265,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int i;
             sObject* new_hash = HASH_NEW_GC(16, TRUE);
 
-            if(!add_object_to_objects_in_pipe(new_hash, runinfo, runinfo))
+            if(!add_object_to_objects_in_pipe(new_hash, runinfo, runinfo->mCommand))
             {
                 return FALSE;
             }
@@ -1312,7 +1312,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 if(sRunInfo_option(runinfo, "-local")) {
                     hash = uobject_item(SFUN(runinfo->mRunningObject).mLocalObjects, runinfo->mArgsRuntime[1]);
 
-                    if(hash == NULL || TYPE(hash) != T_HASH) {
+                    if(hash == NULL || STYPE(hash) != T_HASH) {
                         uobject_put(SFUN(runinfo->mRunningObject).mLocalObjects, runinfo->mArgsRuntime[1], hash = HASH_NEW_GC(16, TRUE));
                     }
                 }
@@ -1325,7 +1325,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                     hash = uobject_item(object, string_c_str(name));
 
-                    if(hash == NULL || TYPE(hash) != T_HASH) {
+                    if(hash == NULL || STYPE(hash) != T_HASH) {
                         if(object == runinfo->mCurrentObject) {
                             uobject_put(runinfo->mCurrentObject, runinfo->mArgsRuntime[1], hash = HASH_NEW_GC(16, TRUE));
                         }
@@ -1420,7 +1420,7 @@ BOOL cmd_object(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 return FALSE;
             }
 
-            if(!add_object_to_objects_in_pipe(new_object, runinfo, runinfo))
+            if(!add_object_to_objects_in_pipe(new_object, runinfo, runinfo->mCommand))
             {
                 return FALSE;
             }
@@ -1446,7 +1446,7 @@ BOOL cmd_object(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             sObject* new_object = UOBJECT_NEW_GC(8, object, "anonymous", TRUE);
             uobject_init(new_object);
 
-            if(!add_object_to_objects_in_pipe(new_object, runinfo, runinfo))
+            if(!add_object_to_objects_in_pipe(new_object, runinfo, runinfo->mCommand))
             {
                 return FALSE;
             }
@@ -1565,7 +1565,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     }
 
                     if(item) {
-                        int type = TYPE(item);
+                        int type = STYPE(item);
 
                         char buf[128];
                         char* obj_kind[T_TYPE_MAX] = {
@@ -1592,7 +1592,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     }
 
                     if(item) {
-                        if(!add_object_to_objects_in_pipe(item, runinfo, runinfo))
+                        if(!add_object_to_objects_in_pipe(item, runinfo, runinfo->mCommand))
                         {
                             return FALSE;
                         }
@@ -1636,7 +1636,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
 
                 sObject* obj = mem;
-                int type = TYPE(obj);
+                int type = STYPE(obj);
 
                 char buf[128];
                 char* obj_kind[T_TYPE_MAX] = {
