@@ -19,7 +19,7 @@ sObject* gStackFrames;
 
 static void object_delete(sObject* obj)
 {
-    switch(TYPE(obj)) {
+    switch(STYPE(obj)) {
     case T_STRING:
         string_delete_on_gc(obj);
         break;
@@ -81,7 +81,7 @@ int object_gc_children_mark(sObject* self)
 {
     int count = 0;
 
-    switch(TYPE(self)) {
+    switch(STYPE(self)) {
     case T_VECTOR:
         count += vector_gc_children_mark(self);
         break;
@@ -134,7 +134,7 @@ static int sweep()
         sObject* p = gPool[i];
         int j;
         for(j=0; j<SLOT_SIZE; j++) {
-           if(TYPE(p + j) != 0 && !IS_MARKED(p + j)) {
+           if(STYPE(p + j) != 0 && !IS_MARKED(p + j)) {
                 object_delete(p + j);
 
                 memset(p + j, 0, sizeof(sObject));
@@ -157,7 +157,7 @@ BOOL gc_valid_object(sObject* object)
         if(object >= gPool[i] && object < gPool[i] + SLOT_SIZE) {
             ulong n = ((ulong)object - (ulong)gPool[i]) % (ulong)sizeof(sObject);
             if(n == 0) {
-                if(TYPE(object) > 0 && TYPE(object) < T_TYPE_MAX) {
+                if(STYPE(object) > 0 && STYPE(object) < T_TYPE_MAX) {
                     return TRUE;
                 }
                 else {
@@ -236,7 +236,7 @@ void gc_final()
         sObject* p = gPool[i];
         int j;
         for(j=0; j<SLOT_SIZE; j++) {
-            if(TYPE(p + j)) {
+            if(STYPE(p + j)) {
                 object_delete(p + j);
             }
         }
