@@ -1,6 +1,7 @@
 #include "config.h"
 #include "xyzsh/xyzsh.h"
 #include <stdio.h>
+#include <string.h>
 
 static void version()
 {
@@ -51,22 +52,23 @@ printf("external_obj %d\n", (int)sizeof(external_obj));
             no_runtime_script = TRUE;
         }
         else {
-            file = argv[i];
+            if(file == NULL) file = argv[i];
         }
     }
 
     if(optc) {
-        xyzsh_init(kATOptC, no_runtime_script);
+        xyzsh_init(kATOptC, TRUE);
         xyzsh_opt_c(optc, argv, argc);
     }
     else if(file) {
-        xyzsh_init(kATOptC, no_runtime_script);
+        xyzsh_init(kATOptC, TRUE);
         (void)xyzsh_load_file(file, argv, argc, gRootObject);
     }
     else {
-        setenv("XYZSH_RUNNING_AS_INTRACTIVE_MODE", "1", 1);
         xyzsh_init(kATConsoleApp, no_runtime_script);
-        xyzsh_readline_interface("", -1, argv, argc, FALSE);
+//int rcode;
+//xyzsh_readline_interface_onetime(&rcode, "ls -al | less", -2, "test", argv, argc, NULL);
+        xyzsh_readline_interface("", -1, argv, argc, FALSE, TRUE);
         xyzsh_kill_all_jobs();
     }
 
