@@ -8,6 +8,7 @@
 #include <unistd.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <ctype.h>
 #include <pwd.h>
 #include <limits.h>
 
@@ -45,8 +46,9 @@ typedef struct {
     BOOL mEnv;
     BOOL mGlob;
     BOOL mTilda;
+    BOOL mQuotedString;
     BOOL mMessagePassing;
-    BOOL mNullString;
+    BOOL mOption;
 } sBuf;
 
 static void add_char_to_buf(sBuf* buf, char c)
@@ -71,7 +73,7 @@ static void add_str_to_buf(sBuf* buf, char* str)
     buf->mLen += len;
 }
 
-static BOOL read_quote(char**p, sBuf* buf, char* sname, int* sline)
+static BOOL read_backslash(char**p, sBuf* buf, char* sname, int* sline)
 {
     (*p)++;
 
@@ -99,6 +101,7 @@ static BOOL read_quote(char**p, sBuf* buf, char* sname, int* sline)
 
         case 'a':
             add_char_to_buf(buf, '\a');
+            
             break;
 
         case '\\':
@@ -115,50 +118,37 @@ static BOOL read_quote(char**p, sBuf* buf, char* sname, int* sline)
 }
 
 static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObject** current_object);
-static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_quote, sCommand* command, sObject* block, sObject** current_object);
+static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCommand* command, sObject* block, sObject** current_object, BOOL read_until_bracket_close);
 
 #define XYZSH_VARIABLE_OBJECT_MAX 8
 
-static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_quote, sCommand* command, sObject* block, sObject** current_object)
+static BOOL read_dollar(char** p, sBuf* buf, char* sname, int* sline, sCommand* command, sObject* block, sObject** current_object)
 {
     (*p)++;
 
     eLineField lf = kLF;
 
-    BOOL double_dollar = FALSE;
-    BOOL option = FALSE;
-
     if(**p == '-') {
         (*p)++;
-        option = TRUE;
+        buf->mOption = TRUE;
     }
 
-    if(**p == '$') {
-        (*p)++;
-        double_dollar = TRUE;
-
-        if(**p == '-') {
+    if(*(*p+1) == '(') {
+        if(**p == 'a') {
             (*p)++;
-            option = TRUE;
+            lf = kBel;
         }
-
-        if(*(*p+1) == '(') {
-            if(**p == 'a') {
-                (*p)++;
-                lf = kBel;
-            }
-            else if(**p == 'm') {
-                (*p)++;
-                lf = kCR;
-            }
-            else if(**p == 'w') {
-                (*p)++;
-                lf = kCRLF;
-            }
-            else if(**p == 'u') {
-                (*p)++;
-                lf = kLF;
-            }
+        else if(**p == 'm') {
+            (*p)++;
+            lf = kCR;
+        }
+        else if(**p == 'w') {
+            (*p)++;
+            lf = kCRLF;
+        }
+        else if(**p == 'u') {
+            (*p)++;
+            lf = kLF;
         }
     }
 
@@ -167,7 +157,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
 
         sObject* block2 = BLOCK_NEW_STACK();
         if(!block_parse(p, sname, sline, block2, current_object)) {
-            (void)sCommand_add_env_block(command, block2, double_dollar, option, lf, sname, *sline);
+            (void)sCommand_add_env_block(command, block2, lf, sname, *sline);
             SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_ENV_BLOCK;
             SBLOCK(block).mCompletionFlags |= command->mEnvsNum & COMPLETION_FLAGS_BLOCK_OR_ENV_NUM;
             return FALSE;
@@ -181,7 +171,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
         add_str_to_buf(buf, buf2);
         add_char_to_buf(buf, PARSER_MAGIC_NUMBER_ENV);
 
-        if(!sCommand_add_env_block(command, block2, double_dollar, option, lf, sname, *sline)) {
+        if(!sCommand_add_env_block(command, block2, lf, sname, *sline)) {
             return FALSE;
         }
     }
@@ -253,11 +243,11 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
         if(**p == '[') {
             (*p)++;
 
-            if(!read_one_argument(p, &key, sname, sline, TRUE, command, block, current_object)) 
+            if(!read_one_argument(p, &key, sname, sline, command, block, current_object, TRUE)) 
             {
                 FREE(name.mBuf);
                 if(key.mEnv) {  // for readline
-                    (void)sCommand_add_arg(command, MANAGED key.mBuf, TRUE, FALSE, sname, *sline);
+                    (void)sCommand_add_arg(command, MANAGED key.mBuf, key.mEnv, key.mQuotedString, key.mGlob, key.mOption, sname, *sline);
                 }
                 else {
                     FREE(key.mBuf);
@@ -285,7 +275,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
         add_str_to_buf(buf, buf2);
         add_char_to_buf(buf, PARSER_MAGIC_NUMBER_ENV);
 
-        if(!sCommand_add_env(command, MANAGED name.mBuf, MANAGED init_value.mBuf, MANAGED key.mBuf, key.mEnv, double_dollar, option, sname, *sline)) {
+        if(!sCommand_add_env(command, MANAGED name.mBuf, MANAGED init_value.mBuf, MANAGED key.mBuf, key.mEnv, key.mQuotedString, sname, *sline)) {
             return FALSE;
         }
     }
@@ -295,7 +285,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
 
 static BOOL add_argument_to_command(MANAGED sBuf* buf, sCommand* command, char* sname, int* sline);
 
-static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_quote, sCommand* command, sObject* block, sObject** current_object)
+static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCommand* command, sObject* block, sObject** current_object, BOOL read_until_bracket_close)
 {
     BOOL squote = FALSE;
     BOOL dquote = FALSE;
@@ -303,26 +293,47 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
         if(!dquote && **p == '\'') {
             (*p)++;
 
-            if(**p == '\'') {
-                (*p)++;
-                buf->mNullString = TRUE;
-            }
-            else {
-                squote = !squote;
-            }
+            buf->mQuotedString = TRUE;
+            squote = !squote;
         }
         else if(!squote && **p == '"') {
             (*p)++;
 
-            if(**p == '"') {
-                (*p)++;
-                buf->mNullString = TRUE;
+            buf->mQuotedString = TRUE;
+            dquote = !dquote;
+        }
+        else if(squote || dquote) {
+            if(**p == 0) {
+                err_msg("close single quote or double quote", sname, *sline, buf->mBuf);
+                return FALSE;
+            }
+            else if(dquote) {
+                if(**p == '\\') {
+                    if(!read_backslash(p, buf, sname, sline)) {
+                        return FALSE;
+                    }
+                }
+                else if(**p == '$') {
+                    if(!read_dollar(p, buf, sname,sline, command, block, current_object)) {
+                        return FALSE;
+                    }
+                }
+                else {
+                    if(**p == '\n') (*sline)++;
+
+                    add_char_to_buf(buf, **p);
+                    (*p)++;
+                }
             }
             else {
-                dquote = !dquote;
+                if(**p == '\n') (*sline)++;
+
+                add_char_to_buf(buf, **p);
+                (*p)++;
             }
         }
-        else if(!dquote && !squote && **p == '%' && (*(*p+1) == 'q' || *(*p+1) == 'Q') && !isalnum(*(*p+2)) && *(*p+2) >= ' ' && *(*p+2) <= 126)
+        /// %q<...> or %Q<...> quote
+        else if(**p == '%' && (*(*p+1) == 'q' || *(*p+1) == 'Q') && !isalnum(*(*p+2)) && *(*p+2) >= ' ' && *(*p+2) <= 126)
         {
             BOOL dquote2 = *(*p+1) == 'Q';
             BOOL delimiter = *(*p+2);
@@ -338,9 +349,35 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
                 delimiter = '}';
             }
 
-            if(**p == delimiter) {
-                (*p)++;
-                buf->mNullString = TRUE;
+            buf->mQuotedString = TRUE;
+
+            if(dquote2) {
+                while(1) {
+                    if(**p == delimiter) {
+                        (*p)++;
+                        break;
+                    }
+                    else if(**p == 0) {
+                        err_msg("reqire to close %q quote",  sname, *sline, buf->mBuf);
+                        return FALSE;
+                    }
+                    else if(**p == '\\') {
+                        if(!read_backslash(p, buf, sname, sline)) {
+                            return FALSE;
+                        }
+                    }
+                    else if(**p == '$') {
+                        if(!read_dollar(p, buf, sname,sline, command, block, current_object)) {
+                            return FALSE;
+                        }
+                    }
+                    else {
+                        if(**p == '\n') (*sline) ++;
+
+                        add_char_to_buf(buf, **p);
+                        (*p)++;
+                    }
+                }
             }
             else {
                 while(1) {
@@ -352,57 +389,23 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
                         err_msg("reqire to close %q quote",  sname, *sline, buf->mBuf);
                         return FALSE;
                     }
-                    else if(dquote2 && **p == '\\') {
-                        if(!read_quote(p, buf, sname, sline)) {
-                            return FALSE;
-                        }
-                    }
-                    else if(dquote2 && **p == '$') {
-                        if(!read_env(p, buf, sname,sline, expand_quote, command, block, current_object)) {
-                            return FALSE;
-                        }
-                    }
-                    else if(**p == '\n') {
-                        (*sline)++;
-                        add_char_to_buf(buf, **p);
-                        (*p)++;
-                    }
                     else {
+                        if(**p == '\n') (*sline) ++;
+
                         add_char_to_buf(buf, **p);
                         (*p)++;
                     }
                 }
             }
         }
-        /// env ///
-        else if(!squote && **p == '$') {
-            if(!read_env(p, buf, sname,sline, expand_quote, command, block, current_object)) {
-                return FALSE;
-            }
-        }
-        else if(squote || dquote) {
-            if(**p == 0) {
-                err_msg("close single quote or double quote", sname, *sline, buf->mBuf);
-                return FALSE;
-            }
-            else if(dquote && **p == '\\') {
-                if(!read_quote(p, buf, sname, sline)) {
-                    return FALSE;
-                }
-            }
-            else if(**p == '\n') {
-                (*sline)++;
-                add_char_to_buf(buf, **p);
-                (*p)++;
-            }
-            else {
-                add_char_to_buf(buf, **p);
-                (*p)++;
-            }
-        }
         /// quote ///
         else if(**p == '\\') {
-            if(!read_quote(p, buf, sname, sline)) {
+            if(!read_backslash(p, buf, sname, sline)) {
+                return FALSE;
+            }
+        }
+        else if(**p == '$') {
+            if(!read_dollar(p, buf, sname,sline, command, block, current_object)) {
                 return FALSE;
             }
         }
@@ -473,7 +476,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
                 }
                 /// env ///
                 else if(!squote && **p == '$') {
-                    if(!read_env(p, buf, sname,sline, expand_quote, command, block, current_object)) {
+                    if(!read_dollar(p, buf, sname,sline, command, block, current_object)) {
                         return FALSE;
                     }
                 }
@@ -513,12 +516,6 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
 
             SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_HERE_DOCUMENT;
         }
-        /// option ///
-        else if(command->mArgsNum > 0 && buf->mLen == 0 && **p == '-' && (isalpha(*(*p+1)) || *(*p+1) == '-' || *(*p+1) == '_'))
-        {
-            add_char_to_buf(buf, PARSER_MAGIC_NUMBER_OPTION);
-            (*p)++;
-        }
         /// tilda ///
         else if(buf->mLen == 0 && **p == '~') {
             SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_TILDA;
@@ -595,47 +592,33 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
 
             skip_spaces(p);
         }
-/*
-        /// redirect error out ///
-        else if(**p == '2' && *(*p+1) == '>') {
-            if(buf->mBuf[0] != 0) {
-                /// message passing ///
-                if(buf->mMessagePassing) {
-                    if(!sCommand_add_message(command, MANAGED buf->mBuf, sname, *sline)) {
-                        return FALSE;
-                    }
-                }
-                else if(!add_argument_to_command(MANAGED buf, command, sname, sline)) {
-                    return FALSE;
-                }
-
-                memset(buf, 0, sizeof(sBuf));
-                buf->mBuf = MALLOC(64);
-                buf->mBuf[0] = 0;
-                buf->mSize = 64;
-            }
-
-            (*p)++;
-
-            if(**p == '>') {
-                buf->mRedirect = REDIRECT_ERROR_APPEND;
-                (*p)++;
-            }
-            else {
-                buf->mRedirect = REDIRECT_ERROR_OUT;
-            }
-
-            skip_spaces(p);
-
-            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_AFTER_REDIRECT;
+        else if(read_until_bracket_close && **p == ']') {
+            break;
         }
-*/
         /// glob ///
-        else if(command->mArgsNum > 0 && (**p == '[' || **p == '?' || **p == '*')) {
+        else if(command->mArgsNum > 0 && (**p == '[' || **p == ']' || **p == '?' || **p == '*')) {
             add_char_to_buf(buf, **p);
             (*p)++;
 
             buf->mGlob = TRUE;
+
+            if(buf->mOption) {
+                err_msg("can't take option with glob", sname, *sline, buf->mBuf);
+                return FALSE;
+            }
+        }
+        /// option ///
+        else if(command->mArgsNum > 0 && buf->mLen == 0 && **p == '-' && (isalpha(*(*p+1)) || *(*p+1) == '-' || *(*p+1) == '_'))
+        {
+            add_char_to_buf(buf, **p);
+            (*p)++;
+
+            buf->mOption = TRUE;
+
+            if(buf->mGlob) {
+                err_msg("can't take option with glob", sname, *sline, buf->mBuf);
+                return FALSE;
+            }
         }
         /// message passing ///
         else if(command->mArgsNum == 0 && **p == ':' && *(*p+1) == ':') {
@@ -797,27 +780,17 @@ static BOOL add_argument_to_command(MANAGED sBuf* buf, sCommand* command, char*
 
     /// redirect ///
     if(buf->mRedirect) {
-        /// glob ///
-        if(buf->mGlob && command->mArgsNum > 0) {
-            if(!sCommand_add_redirect(command, MANAGED buf->mBuf, buf->mEnv, buf->mGlob, buf->mRedirect, sname, *sline)) {
-                return FALSE;
-            }
+        if(buf->mOption) {
+            err_msg("can't take redirect with option", sname, *sline, NULL);
+            return FALSE;
         }
-        else {
-            if(!sCommand_add_redirect(command, MANAGED buf->mBuf, buf->mEnv, FALSE, buf->mRedirect, sname, *sline)) {
-                return FALSE;
-            }
-        }
-    }
-    /// glob ///
-    else if(buf->mGlob && command->mArgsNum > 0) {
-        if(!sCommand_add_arg(command, MANAGED buf->mBuf, buf->mEnv, buf->mGlob, sname, *sline)) {
+        if(!sCommand_add_redirect(command, MANAGED buf->mBuf, buf->mEnv, buf->mQuotedString, buf->mGlob, buf->mRedirect, sname, *sline)) {
             return FALSE;
         }
     }
     /// add buf to command ///
-    else if(buf->mNullString || buf->mBuf[0] != 0) {
-        if(!sCommand_add_arg(command, MANAGED buf->mBuf, buf->mEnv, FALSE, sname, *sline)) {
+    else if(buf->mQuotedString || buf->mBuf[0] != 0) {
+        if(!sCommand_add_arg(command, MANAGED buf->mBuf, buf->mEnv, buf->mQuotedString, buf->mGlob, buf->mOption, sname, *sline)) {
             return FALSE;
         }
     }
@@ -837,10 +810,12 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
         buf.mBuf[0] = 0;
         buf.mSize = 64;
 
-        if(!read_one_argument(p, &buf, sname, sline, TRUE, command, block, current_object)) 
+        SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_AFTER_SPACE;;
+
+        if(!read_one_argument(p, &buf, sname, sline, command, block, current_object, FALSE)) 
         {
             if(buf.mEnv) {  // for readline
-                (void)sCommand_add_arg(command, MANAGED buf.mBuf, TRUE, FALSE, sname, *sline);
+                (void)sCommand_add_arg(command, MANAGED buf.mBuf, TRUE, buf.mQuotedString, buf.mGlob, buf.mOption, sname, *sline);
             }
             else {
                 FREE(buf.mBuf);
@@ -873,7 +848,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
                     return FALSE;
                 }
 
-                if(!sCommand_add_arg(command, MANAGED STRDUP("subshell"), 0, 0, sname, *sline)) {
+                if(!sCommand_add_arg(command, MANAGED STRDUP("subshell"), 0, 0, 0, 0, sname, *sline)) {
                     return FALSE;
                 }
                 if(!sCommand_add_block(command, block2, sname, *sline)) {
@@ -891,17 +866,17 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
                         sObject* current_object2 = *current_object;
                         sObject* object = access_object(command->mMessages[0], &current_object2, NULL);
 
-                        if(object && TYPE(object) == T_UOBJECT) {
+                        if(object && STYPE(object) == T_UOBJECT) {
                             int i;
                             for(i=1; i<command->mMessagesNum; i++) {
                                 object = uobject_item(object, command->mMessages[i]);
 
-                                if(object == NULL || TYPE(object) != T_UOBJECT) {
+                                if(object == NULL || STYPE(object) != T_UOBJECT) {
                                     break;
                                 }
                             }
 
-                            if(object && TYPE(object) == T_UOBJECT) {
+                            if(object && STYPE(object) == T_UOBJECT) {
                                 *current_object = object;
                             }
                         }
@@ -931,6 +906,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
         else if(**p == ' ' || **p == '\t') {
             skip_spaces(p);
             SBLOCK(block).mCompletionFlags &= ~(COMPLETION_FLAGS_INPUTING_COMMAND_NAME|COMPLETION_FLAGS_ENV|COMPLETION_FLAGS_TILDA|COMPLETION_FLAGS_AFTER_REDIRECT|COMPLETION_FLAGS_AFTER_EQUAL);
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_AFTER_SPACE;
         }
         else {
             break;
@@ -1062,6 +1038,11 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
         (*p)++;
         skip_spaces(p);
 
+        if(gAppType == kATOptC) {
+            err_msg("In script mode a job can't run on background. On interactive shell a job can run on background", sname, *sline, NULL);
+            return FALSE;
+        }
+
         statment->mFlags |= STATMENT_FLAGS_BACKGROUND;
         SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_END;
     }
