@@ -19,20 +19,20 @@
 #include <limits.h>
 #include "xyzsh/xyzsh.h"
 
-enum eAppType gAppType;
 sObject* gStdin;
 sObject* gStdout;
 sObject* gStderr;
 
+enum eAppType gAppType;
+eLineField gLineField;
+enum eKanjiCode gKanjiCode;
+
 sObject* gGlobalPipe;
 
-static sObject* gObjectsInPipe;
-
 static sObject* gRunningObjects;
-
 sRunInfo* gRunInfoOfRunningObject;
 
-sObject* gDynamicLibraryFinals;
+static sObject* gObjectsInPipe;
 
 BOOL contained_in_pipe(sObject* object)
 {
@@ -59,54 +59,12 @@ BOOL add_object_to_objects_in_pipe(sObject* object, sRunInfo* runinfo, sCommand*
     return TRUE;
 }
 
-eLineField gLineField;
-enum eKanjiCode gKanjiCode;
-
+//////////////////////////////////////////////////
+/// job
+//////////////////////////////////////////////////
 void (*xyzsh_job_done)(int job_num, char* job_title);
-
 sObject* gJobs;
 
-void run_init(enum eAppType app_type)
-{
-    gAppType = app_type;
-    gStdin = FD_NEW_STACK();
-    gStdout = FD_NEW_STACK();
-    gStderr = FD_NEW_STACK();
-    gJobs = VECTOR_NEW_GC(16, FALSE);
-    uobject_put(gXyzshObject, "_jobs", gJobs);
-
-    xyzsh_job_done = NULL;
-
-    gLineField = kLF;
-    gKanjiCode = kByte;
-
-    gRunInfoOfRunningObject = NULL;
-
-    gGlobalPipe = FD_NEW_STACK();
-
-    gObjectsInPipe = VECTOR_NEW_GC(10, FALSE);
-    uobject_put(gXyzshObject, "_objects_in_pipe", gObjectsInPipe);
-
-    gRunningObjects = VECTOR_NEW_GC(10, FALSE);
-    uobject_put(gXyzshObject, "_running_block", gRunningObjects);
-
-    gDynamicLibraryFinals = HASH_NEW_MALLOC(10);
-}
-
-void run_final()
-{
-    hash_it* it = hash_loop_begin(gDynamicLibraryFinals);
-    while(it) {
-        int (*func)() = hash_loop_item(it);
-        if(func() != 0) {
-            fprintf(stderr, "false in finalize");
-        }
-
-        it = hash_loop_next(it);
-    }
-    hash_delete_on_malloc(gDynamicLibraryFinals);
-}
-
 sObject* job_new_on_gc(char* name, pid_t pgroup, struct termios tty)
 {
     sObject* self = gc_get_free_object(T_JOB, FALSE);
@@ -125,6 +83,165 @@ void job_delete_on_gc(sObject* self)
     if(SJOB(self).mName) FREE(SJOB(self).mName);
 }
 
+BOOL forground_job(int num)
+{
+    if(num>=0 && num < vector_count(gJobs)) {
+        sObject* job = vector_item(gJobs, num);
+
+        tcsetattr(STDIN_FILENO, TCSANOW, SJOB(job).mTty);
+        if(tcsetpgrp(0, SJOB(job).mPGroup) < 0) {
+            perror("tcsetpgrp(fg)");
+            return FALSE;
+        }
+        
+        if(kill(SJOB(job).mPGroup, SIGCONT) < 0) {
+            perror("kill(fg)");
+            return FALSE;
+        }
+
+        /// wait ///
+        int status = 0;
+        pid_t pid = waitpid(SJOB(job).mPGroup, &status, WUNTRACED);
+
+        if(pid < 0) {
+            perror("waitpid(run_wait_cprogs)");
+            return FALSE;
+        }
+
+        /// exited normally ///
+        if(WIFEXITED(status)) {
+            if(xyzsh_job_done) xyzsh_job_done(num, SJOB(job).mName);
+
+            vector_erase(gJobs, num);
+
+            /// forground xyzsh ///
+            mreset_tty();
+            if(tcsetpgrp(0, getpgid(0)) < 0) {
+                perror("tcsetpgrp");
+                return FALSE;
+            }
+        }
+        /// exited from signal ///
+        else if(WIFSIGNALED(status)) {
+            if(xyzsh_job_done) xyzsh_job_done(num, SJOB(job).mName);
+
+            vector_erase(gJobs, num);
+
+            /// froground xyzsh ///
+            if(tcsetpgrp(0, getpgid(0)) < 0) {
+                perror("tcsetpgrp");
+                return FALSE;
+            }
+        }
+        /// stopped from signal ///
+        else if(WIFSTOPPED(status)) {
+            tcgetattr(STDIN_FILENO, SJOB(job).mTty);
+
+            /// forground xyzsh ///
+            if(tcsetpgrp(0, getpgid(0)) < 0) {
+                perror("tcsetpgrp");
+                return FALSE;
+            }
+        }
+
+        return TRUE;
+    }
+
+    err_msg("invalid job number", "fg", 1, "fg");
+    return FALSE;
+}
+
+BOOL background_job(int num)
+{
+    if(num>=0 && num < vector_count(gJobs)) {
+        sObject* job = vector_item(gJobs, num);
+        
+        if(kill(SJOB(job).mPGroup, SIGCONT) < 0) {
+            perror("kill(bg)");
+            return FALSE;
+        }
+    }
+
+    return TRUE;
+}
+
+void xyzsh_wait_background_job()
+{
+    int status;
+    pid_t pid = waitpid(-1, &status, WNOHANG);
+
+    if(pid >= 0) {
+        int i;
+        for(i=0; i<vector_count(gJobs); i++) {
+            sObject* job = vector_item(gJobs, i);
+
+            if(pid == SJOB(job).mPGroup) {
+                if(xyzsh_job_done) xyzsh_job_done(i, SJOB(job).mName);
+
+                vector_erase(gJobs, i);
+                break;
+            }
+        }
+    }
+}
+
+void xyzsh_kill_all_jobs()
+{
+    int i;
+    for(i=0; i<vector_count(gJobs); i++) {
+        sObject* job = vector_item(gJobs, i);
+
+        kill(SJOB(job).mPGroup, SIGKILL);
+    }
+
+    vector_clear(gJobs);
+}
+
+static void make_job_title(sRunInfo* runinfo, char* title, int title_num)
+{
+    title[0] = 0;
+
+    sStatment* statment = runinfo->mStatment;
+    sCommand* command_ = runinfo->mCommand;
+
+    int i;
+    for(i=0; i<statment->mCommandsNum; i++) {
+        sCommand* command = statment->mCommands + i;
+
+        if(command == command_) {
+            int j;
+            for(j=0; j<runinfo->mArgsNumRuntime; j++) {
+                char* arg = runinfo->mArgsRuntime[j];
+
+                xstrncat(title, arg, title_num);
+                if(j+1 < runinfo->mArgsNumRuntime) {
+                    xstrncat(title, " ", title_num);
+                }
+            }
+            break;
+        }
+        else {
+            int j;
+            for(j=0; j<command->mArgsNum; j++) {
+                char* arg = command->mArgs[j];
+
+                xstrncat(title, arg, title_num);
+
+                if(j+1 < command->mArgsNum) {
+                    xstrncat(title, " ", title_num);
+                }
+            }
+        }
+
+        if(i+1 < statment->mCommandsNum) {
+            xstrncat(title, "|", title_num);
+        }
+    }
+}
+
+///////////////////////////////////////////////////////////////////////////////
+// run external program
+///////////////////////////////////////////////////////////////////////////////
 static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo, char* program)
 {
     char buf[BUFSIZ+1];
@@ -209,58 +326,6 @@ static void nextin_writer(pid_t pid, sObject* nextin, int* pipeinfds, int* pipeo
     *nextin_reader_pid = pid2;
 }
 
-static void make_job_title(sRunInfo* runinfo, char* title, int title_num)
-{
-    title[0] = 0;
-
-    sStatment* statment = runinfo->mStatment;
-    sCommand* command_ = runinfo->mCommand;
-
-    int i;
-    for(i=0; i<statment->mCommandsNum; i++) {
-        sCommand* command = statment->mCommands + i;
-
-        if(command == command_) {
-            int j;
-            for(j=0; j<runinfo->mArgsNumRuntime; j++) {
-                char* arg = runinfo->mArgsRuntime[j];
-
-                xstrncat(title, arg, title_num);
-                if(j+1 < runinfo->mArgsNumRuntime) {
-                    xstrncat(title, " ", title_num);
-                }
-            }
-            break;
-        }
-        else {
-            int j;
-            for(j=0; j<command->mArgsNum; j++) {
-                char* arg = command->mArgs[j];
-
-                char buf[16];
-                buf[0] = PARSER_MAGIC_NUMBER_ENV;
-                buf[1] = 0;
-
-                if(arg[0] == PARSER_MAGIC_NUMBER_OPTION || strstr(arg, buf)) {
-                    if(j+1 < command->mArgsNum) {
-                        xstrncat(title, " ", title_num);
-                    }
-                }
-                else {
-                    xstrncat(title, arg, title_num);
-                    if(j+1 < command->mArgsNum) {
-                        xstrncat(title, " ", title_num);
-                    }
-                }
-            }
-        }
-
-        if(i+1 < statment->mCommandsNum) {
-            xstrncat(title, "|", title_num);
-        }
-    }
-}
-
 static BOOL wait_child_program(pid_t pid, pid_t nextin_reader_pid, int nextout2, sRunInfo* runinfo, char* program)
 {
     if(nextin_reader_pid != -1) {
@@ -521,7 +586,7 @@ static BOOL run_external_command(char* program, sObject* nextin, sObject* nextou
     if(nextin == gStdin && SFD(gStdin).mBufLen == 0) {
         nextin2 = 0;
     }
-    else if(TYPE(nextin) == T_FD2) {
+    else if(STYPE(nextin) == T_FD2) {
         nextin2 = SFD2(nextin).mFD;
     }
     else {
@@ -535,7 +600,7 @@ static BOOL run_external_command(char* program, sObject* nextin, sObject* nextou
         nextout2 = 1;
         nexterr2 = 2;
     }
-    else if(TYPE(nextout) == T_FD2) {
+    else if(STYPE(nextout) == T_FD2) {
         nextout2 = SFD2(nextout).mFD;
 
         if(pipe(pipeerrfds) < 0) {
@@ -621,10 +686,10 @@ if(1) {
         /// Therefore xyzsh can't waitpid and deadlock.
         /// Avoid this, xyzsh is ready for a writer process to write buffer to the xyzsh.
         pid_t nextin_reader_pid = -1;
-        if(TYPE(nextin) != T_FD2 && nextin2 != 0) {
+        if(STYPE(nextin) != T_FD2 && nextin2 != 0) {
             nextin_writer(pgroup, nextin, pipeinfds, pipeoutfds, pipeerrfds, runinfo, program, &nextin_reader_pid);
         }
-        if(TYPE(nextout) != T_FD2 && nextout2 != 1) {
+        if(STYPE(nextout) != T_FD2 && nextout2 != 1) {
             (void)close(pipeoutfds[1]);
             if(!nextout_reader(nextout, pipeoutfds, runinfo, program)) {
                 // wait everytime
@@ -649,6 +714,9 @@ if(1) {
     }
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// run an object
+///////////////////////////////////////////////////////////////////////////////
 BOOL run_function(sObject* fun, sObject* nextin, sObject* nextout, sRunInfo* runinfo, char** argv, int argc, sObject** blocks, int blocks_num)
 {
     sObject* local_objects = SFUN(fun).mLocalObjects;
@@ -751,7 +819,7 @@ static BOOL run_block(sObject* block, sObject* nextin, sObject* nextout, sRunInf
 
 BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
-    switch(TYPE(object)) {
+    switch(STYPE(object)) {
         case T_NFUN: 
             if(gAppType == kATConsoleApp)
             {
@@ -933,6 +1001,9 @@ BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunInfo* ru
     return TRUE;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// statment tree
+///////////////////////////////////////////////////////////////////////////////
 static BOOL stdin_read_out()
 {
     char buf[BUFSIZ+1];
@@ -981,7 +1052,7 @@ static BOOL ready_for_redirect(sObject** nextout, sObject** nextin, int* opened_
                 }
                 (void)close(fd);
 
-                ASSERT(TYPE(*nextin) == T_FD);
+                ASSERT(STYPE(*nextin) == T_FD);
                 fd_put(*nextin, MANAGED result, result_size, result_len);
                 break;
 
@@ -1017,6 +1088,24 @@ static BOOL ready_for_redirect(sObject** nextout, sObject** nextin, int* opened_
         }
     }
 
+    if(*nextout == NULL) {
+        if(last_program) {
+            if(statment->mFlags & STATMENT_FLAGS_GLOBAL_PIPE_OUT) {
+                fd_clear(gGlobalPipe);
+                *nextout = gGlobalPipe;
+            }
+            else if(statment->mFlags & STATMENT_FLAGS_GLOBAL_PIPE_APPEND) {
+                *nextout = gGlobalPipe;
+            }
+            else {
+                *nextout = pipeout;
+            }
+        }
+        else {
+            *nextout = FD_NEW_STACK();
+        }
+    }
+
     return TRUE;
 }
 
@@ -1025,16 +1114,50 @@ static void redirect_finish(int opend_fd)
     (void)close(opend_fd);
 }
 
-static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout, sRunInfo* runinfo, sObject* current_object)
+static BOOL inherit_method(sRunInfo* runinfo, sObject** object, sCommand* command)
 {
-    /// error check ///
-    if(statment->mFlags & STATMENT_FLAGS_BACKGROUND) {
-        if(gAppType == kATOptC) {
-            err_msg("can't make a job background on script mode. you can make a job background on interactive shell", runinfo->mSName, runinfo->mSLine, NULL);
+    sObject* running_object = runinfo->mRunningObject;
+
+    if(runinfo->mRunningObject) {
+        switch(STYPE(running_object)) {
+        case T_FUN:  {
+            sObject* parent = SFUN(running_object).mParent;
+
+            if(parent) {
+                if(STYPE(parent) == T_NFUN || STYPE(parent) == T_FUN) {
+                    *object = parent;
+                }
+            }
+            }
+            break;
+
+        case T_CLASS: {
+            sObject* parent = SFUN(running_object).mParent;
+
+            if(parent) {
+                if(STYPE(parent) == T_CLASS) {
+                    *object = parent;
+                }
+            }
+            }
+            break;
+
+        case T_NFUN:
+        case T_BLOCK:
+            err_msg("There is not a parent object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             return FALSE;
+
+        default:
+            fprintf(stderr, "unexpected error in cmd_inherit\n");
+            exit(1);
         }
     }
 
+    return TRUE;
+}
+
+static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout, sRunInfo* runinfo, sObject* current_object)
+{
     /// head of statment ///
     sObject* nextin;
     if(statment->mFlags & STATMENT_FLAGS_CONTEXT_PIPE) {
@@ -1119,50 +1242,14 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
 
         sRunInfo_command_new(runinfo);
 
-        /// expand env ///
-        if(!sCommand_expand_env_of_redirect(command, pipein, runinfo))
-        {
-            sRunInfo_command_delete(runinfo);
-            return FALSE;
-        }
-
+        /// run a command ///
         sObject* nextout = NULL;
-
         int opened_fd = -1;
-        if(!ready_for_redirect(&nextout, &nextin, &opened_fd, pipeout, last_program, statment, command, runinfo)) {
-            sRunInfo_command_delete(runinfo);
-            return FALSE;;
-        }
-        
-        ASSERT(TYPE(nextin) == T_FD);
-
-        if(nextout == NULL) {
-            if(last_program) {
-                if(statment->mFlags & STATMENT_FLAGS_GLOBAL_PIPE_OUT) {
-                    fd_clear(gGlobalPipe);
-                    nextout = gGlobalPipe;
-                }
-                else if(statment->mFlags & STATMENT_FLAGS_GLOBAL_PIPE_APPEND) {
-                    nextout = gGlobalPipe;
-                }
-                else {
-                    nextout = pipeout;
-                }
-            }
-            else {
-                nextout = FD_NEW_STACK();
-            }
-        }
 
         if(command->mArgsNum > 0) {
-            if(command->mArgsFlags[0] & XYZSH_ARGUMENT_ENV) {
-                err_msg("a command name must be determined staticaly. Instead of this, use eval inner command.", runinfo->mSName, runinfo->mSLine, NULL);
-                sRunInfo_command_delete(runinfo);
-                return FALSE;
-            }
-
-            /// message passing ///
+            /// message passing with object prefix ///
             if(command->mMessagesNum > 0) {
+                /// search for object ///
                 sObject* reciever = current_object;
                 sObject* object;
                 if(command->mMessages[0][0] == 0) {
@@ -1173,7 +1260,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                 }
                 int i = 1;
                 while(i < command->mMessagesNum) {
-                    if(object && TYPE(object) == T_UOBJECT) {
+                    if(object && STYPE(object) == T_UOBJECT) {
                         reciever = object;
                         object = uobject_item(object, command->mMessages[i++]);
                     }
@@ -1184,57 +1271,34 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                     }
                 }
 
-                if(object && TYPE(object) == T_UOBJECT) {
+                if(object && STYPE(object) == T_UOBJECT) {
                     reciever = object;
                     object = uobject_item(object, command->mArgs[0]);
 
-                    if(object == gInheritObject) {
-                        sObject* running_object = runinfo->mRunningObject;
-
-                        if(runinfo->mRunningObject) {
-                            switch(TYPE(running_object)) {
-                            case T_FUN:  {
-                                sObject* parent = SFUN(running_object).mParent;
-
-                                if(parent) {
-                                    if(TYPE(parent) == T_NFUN || TYPE(parent) == T_FUN) {
-                                        object = parent;
-                                    }
-                                }
-                                }
-                                break;
-
-                            case T_CLASS: {
-                                sObject* parent = SFUN(running_object).mParent;
-
-                                if(parent) {
-                                    if(TYPE(parent) == T_CLASS) {
-                                        object = parent;
-                                    }
-                                }
-                                }
-                                break;
-
-                            case T_NFUN:
-                            case T_BLOCK:
-                                err_msg("There is not a parent object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                                sRunInfo_command_delete(runinfo);
-                                return FALSE;
-
-                            default:
-                                fprintf(stderr, "unexpected error in cmd_inherit\n");
-                                exit(1);
-                            }
+                    /// inherit object ///
+                    if(object == gInheritMethod) {
+                        if(!inherit_method(runinfo, &object, command)) {
+                            sRunInfo_command_delete(runinfo);
+                            return FALSE;
                         }
                     }
 
                     /// expand env ///
-                    if(!sCommand_expand_env(command, object, pipein, nextout, runinfo))
+                    if(!sCommand_expand_env(command, object, pipein, runinfo))
                     {
                         sRunInfo_command_delete(runinfo);
                         return FALSE;
                     }
 
+                    /// redirect ///
+                    if(!ready_for_redirect(&nextout, &nextin, &opened_fd, pipeout, last_program, statment, command, runinfo)) {
+                        sRunInfo_command_delete(runinfo);
+                        return FALSE;;
+                    }
+                    
+                    ASSERT(STYPE(nextin) == T_FD);
+
+                    /// run object ///
                     if(object) {
                         runinfo->mRecieverObject = reciever;
                         if(!run_object(object, nextin, nextout, runinfo)) {
@@ -1254,59 +1318,36 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                     return FALSE;
                 }
             }
-            /// normal function ///
+            /// message passing without object prefix ///
             else {
                 sObject* reciever = current_object;
                 sObject* object = access_object(command->mArgs[0], &reciever, runinfo->mRunningObject);
 
-                if(object == gInheritObject) {
-                    sObject* running_object = runinfo->mRunningObject;
-
-                    if(runinfo->mRunningObject) {
-                        switch(TYPE(running_object)) {
-                        case T_FUN:  {
-                            sObject* parent = SFUN(running_object).mParent;
-
-                            if(parent) {
-                                if(TYPE(parent) == T_NFUN || TYPE(parent) == T_FUN) {
-                                    object = parent;
-                                }
-                            }
-                            }
-                            break;
-
-                        case T_CLASS: {
-                            sObject* parent = SFUN(running_object).mParent;
-
-                            if(parent) {
-                                if(TYPE(parent) == T_CLASS) {
-                                    object = parent;
-                                }
-                            }
-                            }
-                            break;
-
-                        case T_NFUN:
-                        case T_BLOCK:
-                            err_msg("There is not a parent object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                            sRunInfo_command_delete(runinfo);
-                            return FALSE;
-
-                        default:
-                            fprintf(stderr, "unexpected error in cmd_inherit\n");
-                            exit(1);
-                        }
-                    }
-                }
-
-                if(object) {
-                    /// expand env ///
-                    if(!sCommand_expand_env(command, object, pipein, nextout, runinfo))
-                    {
+                /// inherit method ///
+                if(object == gInheritMethod) {
+                    if(!inherit_method(runinfo, &object, command)) {
                         sRunInfo_command_delete(runinfo);
                         return FALSE;
                     }
+                }
 
+                /// expand env ///
+                if(!sCommand_expand_env(command, object, pipein, runinfo))
+                {
+                    sRunInfo_command_delete(runinfo);
+                    return FALSE;
+                }
+
+                /// redirect ///
+                if(!ready_for_redirect(&nextout, &nextin, &opened_fd, pipeout, last_program, statment, command, runinfo)) {
+                    sRunInfo_command_delete(runinfo);
+                    return FALSE;;
+                }
+                
+                ASSERT(STYPE(nextin) == T_FD);
+
+                if(object) {
+                    /// run object ///
                     runinfo->mRecieverObject = reciever;
                     if(!run_object(object, nextin, nextout, runinfo)) {
                         sRunInfo_command_delete(runinfo);
@@ -1314,13 +1355,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                     }
                 }
                 else {
-                    /// expand env ///
-                    if(!sCommand_expand_env(command, NULL, pipein, nextout, runinfo))
-                    {
-                        sRunInfo_command_delete(runinfo);
-                        return FALSE;
-                    }
-
+                    /// run external object ///
                     if(!run_external_command(runinfo->mArgsRuntime[0], nextin, nextout, runinfo)) {
                         sRunInfo_command_delete(runinfo);
                         return FALSE;
@@ -1331,7 +1366,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
 
         if(opened_fd != -1) redirect_finish(opened_fd);
 
-        if(TYPE(nextout) == T_FD2) {
+        if(STYPE(nextout) == T_FD2) {
             nextin = FD_NEW_STACK();
         }
         else {
@@ -1349,6 +1384,9 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
     return TRUE;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// run 
+///////////////////////////////////////////////////////////////////////////////
 BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject* current_object, sObject* running_object)
 {
     vector_add(gRunningObjects, running_object); // for marking on gc
@@ -1517,309 +1555,31 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
     return TRUE;
 }
 
-BOOL forground_job(int num)
+void run_init(enum eAppType app_type)
 {
-    if(num>=0 && num < vector_count(gJobs)) {
-        sObject* job = vector_item(gJobs, num);
+    gAppType = app_type;
+    gStdin = FD_NEW_STACK();
+    gStdout = FD_NEW_STACK();
+    gStderr = FD_NEW_STACK();
+    gJobs = VECTOR_NEW_GC(16, FALSE);
+    uobject_put(gXyzshObject, "_jobs", gJobs);
 
-        tcsetattr(STDIN_FILENO, TCSANOW, SJOB(job).mTty);
-        if(tcsetpgrp(0, SJOB(job).mPGroup) < 0) {
-            perror("tcsetpgrp(fg)");
-            return FALSE;
-        }
-        
-        if(kill(SJOB(job).mPGroup, SIGCONT) < 0) {
-            perror("kill(fg)");
-            return FALSE;
-        }
+    xyzsh_job_done = NULL;
 
-        /// wait ///
-        int status = 0;
-        pid_t pid = waitpid(SJOB(job).mPGroup, &status, WUNTRACED);
+    gLineField = kLF;
+    gKanjiCode = kByte;
 
-        if(pid < 0) {
-            perror("waitpid(run_wait_cprogs)");
-            return FALSE;
-        }
+    gRunInfoOfRunningObject = NULL;
 
-        /// exited normally ///
-        if(WIFEXITED(status)) {
-            if(xyzsh_job_done) xyzsh_job_done(num, SJOB(job).mName);
+    gGlobalPipe = FD_NEW_STACK();
 
-            vector_erase(gJobs, num);
+    gObjectsInPipe = VECTOR_NEW_GC(10, FALSE);
+    uobject_put(gXyzshObject, "_objects_in_pipe", gObjectsInPipe);
 
-            /// forground xyzsh ///
-            mreset_tty();
-            if(tcsetpgrp(0, getpgid(0)) < 0) {
-                perror("tcsetpgrp");
-                return FALSE;
-            }
-        }
-        /// exited from signal ///
-        else if(WIFSIGNALED(status)) {
-            if(xyzsh_job_done) xyzsh_job_done(num, SJOB(job).mName);
-
-            vector_erase(gJobs, num);
-
-            /// froground xyzsh ///
-            if(tcsetpgrp(0, getpgid(0)) < 0) {
-                perror("tcsetpgrp");
-                return FALSE;
-            }
-        }
-        /// stopped from signal ///
-        else if(WIFSTOPPED(status)) {
-            tcgetattr(STDIN_FILENO, SJOB(job).mTty);
-
-            /// forground xyzsh ///
-            if(tcsetpgrp(0, getpgid(0)) < 0) {
-                perror("tcsetpgrp");
-                return FALSE;
-            }
-        }
-
-        return TRUE;
-    }
-
-    err_msg("invalid job number", "fg", 1, "fg");
-    return FALSE;
+    gRunningObjects = VECTOR_NEW_GC(10, FALSE);
+    uobject_put(gXyzshObject, "_running_block", gRunningObjects);
 }
 
-BOOL background_job(int num)
+void run_final()
 {
-    if(num>=0 && num < vector_count(gJobs)) {
-        sObject* job = vector_item(gJobs, num);
-        
-        if(kill(SJOB(job).mPGroup, SIGCONT) < 0) {
-            perror("kill(bg)");
-            return FALSE;
-        }
-    }
-
-    return TRUE;
 }
-
-void xyzsh_wait_background_job()
-{
-    int status;
-    pid_t pid = waitpid(-1, &status, WNOHANG);
-
-    if(pid >= 0) {
-        int i;
-        for(i=0; i<vector_count(gJobs); i++) {
-            sObject* job = vector_item(gJobs, i);
-
-            if(pid == SJOB(job).mPGroup) {
-                if(xyzsh_job_done) xyzsh_job_done(i, SJOB(job).mName);
-
-                vector_erase(gJobs, i);
-                break;
-            }
-        }
-    }
-}
-
-void xyzsh_kill_all_jobs()
-{
-    int i;
-    for(i=0; i<vector_count(gJobs); i++) {
-        sObject* job = vector_item(gJobs, i);
-
-        kill(SJOB(job).mPGroup, SIGKILL);
-    }
-
-    vector_clear(gJobs);
-}
-
-// TRUE: Success. result is malloced.
-// FALSE: signal interrupt. set RCODE_SIGNAL_INTERRUPT on rcode and err message. result is not malloced.
-BOOL bufsiz_read(int fd, ALLOC char** result, int* result_len, int* result_size)
-{
-    int source_size = BUFSIZ * 8;
-    int source_len = 0;
-    char* source = MALLOC(BUFSIZ*8);
-    source[0] = 0;
-
-    char buf[BUFSIZ+1];
-    while(TRUE) {
-        int size = read(fd, buf, BUFSIZ);
-        if(size < 0) {
-            FREE(source);
-            return FALSE;
-        }
-        buf[size] = 0;
-        if(size < BUFSIZ) {
-            if(source_len + size + 1 >= source_size) {
-                source_size *= 2;
-                source = REALLOC(source, source_size);
-            }
-            strcat(source, buf);
-            source_len += size;
-            break;
-        }
-        else {
-            if(source_len + size + 1 >= source_size) {
-                source_size *= 2;
-                source = REALLOC(source, source_size);
-            }
-            strcat(source, buf);
-            source_len += size;
-        }
-    }
-
-    *result = source;
-    *result_len = source_len;
-    *result_size = source_size;
-    
-    return TRUE;
-}
-
-BOOL load_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo, char** argv, int argc)
-{
-    int fd = open(fname, O_RDONLY);
-    if(fd < 0) {
-        err_msg("open error", runinfo->mSName, runinfo->mSLine, fname);
-        return FALSE;
-    }
-
-    char* buf;
-    int buf_len;
-    int buf_size;
-    if(!bufsiz_read(fd, ALLOC &buf, &buf_len, &buf_size)) {
-        sCommand* command = runinfo->mCommand;
-        err_msg("signal interrupt16", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-        close(fd);
-        return FALSE;
-    }
-    (void)close(fd);
-
-    stack_start_stack();
-    int rcode = 0;
-    sObject* block = BLOCK_NEW_STACK();
-    int sline = 1;
-    if(parse(buf, fname, &sline, block, NULL)) {
-        FREE(buf);
-
-        sObject* fun = FUN_NEW_STACK(NULL);
-        sObject* stackframe = UOBJECT_NEW_GC(8, gXyzshObject, "_stackframe", FALSE);
-        vector_add(gStackFrames, stackframe);
-        //uobject_init(stackframe);
-        SFUN(fun).mLocalObjects = stackframe;
-
-        sObject* argv2 = VECTOR_NEW_GC(16, FALSE);
-        int i;
-        for(i=0; i<argc; i++) {
-            vector_add(argv2, STRING_NEW_GC(argv[i], FALSE));
-        }
-        uobject_put(SFUN(fun).mLocalObjects, "ARGV", argv2);
-
-        if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, fun))
-        {
-            if(rcode == RCODE_EXIT) {
-            }
-            else {
-                runinfo->mRCode = rcode;
-
-                (void)vector_pop_back(gStackFrames);
-                stack_end_stack();
-                return FALSE;
-            }
-        }
-        (void)vector_pop_back(gStackFrames);
-    }
-    else {
-        FREE(buf);
-        sCommand* command = runinfo->mCommand;
-        err_msg_adding("parser error", runinfo->mSName, runinfo->mSLine, "parser");
-        stack_end_stack();
-        return FALSE;
-    }
-
-    stack_end_stack();
-
-    return TRUE;
-}
-
-#if defined(__CYGWIN__) && defined(HAVE_MIGEMO_H)
-int migemo_dl_final();
-int migemo_dl_init();
-#endif
-
-BOOL load_so_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    sCommand* command = runinfo->mCommand;
-
-    char path[PATH_MAX];
-    snprintf(path, PATH_MAX, "%s%s", EXTDIR, fname);
-
-    if(access(path, R_OK) != 0) {
-        char* home = getenv("HOME");
-        if(home) {
-            snprintf(path, PATH_MAX, "%s/.xyzsh/lib/%s", home, fname);
-            if(access(path, R_OK) != 0) {
-                xstrncpy(path, fname, PATH_MAX);
-            }
-        }
-        else {
-            xstrncpy(path, fname, PATH_MAX);
-        }
-    }
-
-#if defined(__CYGWIN__) && defined(HAVE_MIGEMO_H)
-    if(hash_item(gDynamicLibraryFinals, path) == NULL) {
-        if(strstr(path, "migemo")) {
-            if(migemo_dl_init() != 0) {
-                err_msg("false in initialize the dynamic library", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                return FALSE;
-            }
-            hash_put(gDynamicLibraryFinals, path, migemo_dl_final);
-        }
-
-        char path2[PATH_MAX];
-        snprintf(path2, PATH_MAX, "%s.xyzsh", path);
-        if(access(path2, F_OK) == 0) {
-            if(!load_file(path2, nextin, nextout, runinfo, NULL, 0)) {
-                return FALSE;
-            }
-        }
-    }
-#else
-    if(hash_item(gDynamicLibraryFinals, path) == NULL) {
-        void* handle = dlopen(path, RTLD_LAZY);
-        if(handle == NULL) {
-            err_msg(dlerror(), runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-            return FALSE;
-        }
-
-        int (*init_func)() = dlsym(handle, "dl_init");
-        int (*final_func)() = dlsym(handle, "dl_final");
-        if(init_func == NULL || final_func == NULL) {
-            err_msg("not found dl_init or dl_final", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-            dlclose(handle);
-            return FALSE;
-        }
-
-        if(init_func(NULL) != 0) {
-            err_msg("false in initialize the dynamic library", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-            dlclose(handle);
-            return FALSE;
-        }
-
-        hash_put(gDynamicLibraryFinals, path, final_func);
-
-        //dlclose(handle);
-
-        char path2[PATH_MAX];
-        snprintf(path2, PATH_MAX, "%s.xyzsh", path);
-        if(access(path2, F_OK) == 0) {
-            if(!load_file(path2, nextin, nextout, runinfo, NULL, 0)) {
-                return FALSE;
-            }
-        }
-    }
-#endif
-
-    return TRUE;
-}
-
