@@ -19,7 +19,7 @@ static int gStackFrameSlot[MAX_STACK];
 
 static void object_delete(sObject* obj) 
 {
-    switch(TYPE(obj)) {
+    switch(STYPE(obj)) {
         case T_STRING:
             string_delete_on_stack(obj);
             break;
