@@ -108,7 +108,7 @@ sObject* gInheritObject;
 
 void uobject_root_init(sObject* self)
 {
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
 
     uobject_put(self, "unset", NFUN_NEW_GC(cmd_unset, NULL, TRUE));
     uobject_put(self, "completion", NFUN_NEW_GC(cmd_completion, NULL, TRUE));
@@ -283,7 +283,7 @@ void uobject_root_init(sObject* self)
     (void)nfun_put_option_with_argument(nfun, STRDUP("-option-with-argument"));
     uobject_put(self, "class", nfun);
 
-    readline_object_init(self);
+    editline_object_init(self);
     curses_object_init(self);
 
     clear_matching_info_variable();
@@ -291,7 +291,7 @@ void uobject_root_init(sObject* self)
 
 void uobject_init(sObject* self)
 {
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
 
     uobject_put(self, "run", NFUN_NEW_GC(cmd_mrun, NULL, FALSE));
     uobject_put(self, "self", self);
@@ -302,7 +302,7 @@ void uobject_init(sObject* self)
 
 void uobject_delete_on_gc(sObject* self)
 {
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
 
    uobject_it* it = SUOBJECT(self).mEntryIt;
 
@@ -318,7 +318,7 @@ void uobject_delete_on_gc(sObject* self)
 
 void uobject_delete_on_stack(sObject* self)
 {
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
 
    uobject_it* it = SUOBJECT(self).mEntryIt;
 
@@ -334,7 +334,7 @@ void uobject_delete_on_stack(sObject* self)
 
 int uobject_gc_children_mark(sObject* self)
 {
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
 
     int count = 0;
 
@@ -357,7 +357,7 @@ int uobject_gc_children_mark(sObject* self)
 
 void uobject_put(sObject* self, char* key, void* item)
 {
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
 
     if(SUOBJECT(self).mCounter >= SUOBJECT(self).mTableSize) {
         resize(self);
@@ -384,7 +384,7 @@ void uobject_put(sObject* self, char* key, void* item)
 
 void* uobject_item(sObject* self, char* key)
 {
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
 
    uobject_it* it = SUOBJECT(self).mTable[ get_uobject_value(self, key) ];
 
@@ -398,7 +398,7 @@ void* uobject_item(sObject* self, char* key)
 
 void uobject_clear(sObject* self)
 {
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
 
    int i;
    int max;
@@ -418,7 +418,7 @@ void uobject_clear(sObject* self)
 
 uobject_it* uobject_loop_begin(sObject* self) 
 { 
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
 
     return SUOBJECT(self).mEntryIt; 
 }
@@ -440,14 +440,14 @@ uobject_it* uobject_loop_next(uobject_it* it)
 
 int uobject_count(sObject* self)
 {
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
 
    return SUOBJECT(self).mCounter;
 }
 
 static void erase_from_list(sObject* self, uobject_it* rit)
 {
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
 
    if(rit == SUOBJECT(self).mEntryIt) {
       SUOBJECT(self).mEntryIt = rit->mNextIt;
@@ -468,7 +468,7 @@ static void erase_from_list(sObject* self, uobject_it* rit)
 
 BOOL uobject_erase(sObject* self, char* key)
 {
-    ASSERT(TYPE(self) == T_UOBJECT);
+    ASSERT(STYPE(self) == T_UOBJECT);
    const unsigned int hash_value = get_uobject_value(self, key);
    uobject_it* it = SUOBJECT(self).mTable[hash_value];
    
