@@ -83,9 +83,17 @@ static void xyzsh_read_rc()
     xyzsh_read_rc_core(user_rc_path);
 }
 
+static void xyzsh_read_rc_mini()
+{
+    char sys_rc_path[PATH_MAX];
+    snprintf(sys_rc_path, PATH_MAX, "%sxyzsh.xyzsh", SYSCONFDIR);
+
+    xyzsh_read_rc_core(sys_rc_path);
+}
+
 void xyzsh_init(enum eAppType app_type, BOOL no_runtime_script)
 {
-    setenv("XYZSH_VERSION", "1.4.2", 1);
+    setenv("XYZSH_VERSION", "1.4.3", 1);
     setenv("XYZSH_DOCDIR", DOCDIR, 1);
     setenv("XYZSH_DATAROOTDIR", DOCDIR, 1);
     setenv("XYZSH_EXT_PATH", EXTDIR, 1);
@@ -142,7 +150,7 @@ void xyzsh_init(enum eAppType app_type, BOOL no_runtime_script)
 
     gc_init(1);
     run_init(app_type);
-    xyzsh_readline_init(FALSE);
+    load_init();
 
     gDirStack = VECTOR_NEW_GC(10, FALSE);
     uobject_put(gXyzshObject, "_dir_stack", gDirStack);
@@ -160,15 +168,21 @@ void xyzsh_init(enum eAppType app_type, BOOL no_runtime_script)
 
     if(!no_runtime_script) {
         xyzsh_read_rc();
-        readline_read_history();
+        //readline_read_history();
     }
+    else {
+        xyzsh_read_rc_mini();
+    }
+
+    xyzsh_editline_init(); // must be after runtime script
 }
 
 void xyzsh_final()
 {
-    readline_write_history();
+    xyzsh_editline_final();
 
     mcurses_final();
+    load_final();
     run_final();
     gc_final();
 
