@@ -174,8 +174,8 @@ typedef struct _option_hash_it {
 
 #define ENV_FLAGS_KIND_ENV 0x01
 #define ENV_FLAGS_KIND_BLOCK 0x02
-#define ENV_FLAGS_DOUBLE_DOLLAR 0x10
-#define ENV_FLAGS_OPTION 0x20
+#define ENV_FLAGS_KEY_ENV 0x10
+#define ENV_FLAGS_KEY_QUOTED_STRING 0x20
 
 typedef unsigned char eEnvKind;
 
@@ -187,7 +187,6 @@ typedef struct {
             char* mName;
             char* mInitValue;
             char* mKey;
-            uchar mKeyEnv;
         };
         struct {
             struct _sObject* mBlock;
@@ -204,9 +203,12 @@ typedef struct {
 #define REDIRECT_ERROR_OUT 0x10
 #define REDIRECT_ENV 0x100
 #define REDIRECT_GLOB 0x200
+#define REDIRECT_QUOTED 0x400
 
 #define XYZSH_ARGUMENT_ENV 0x01
 #define XYZSH_ARGUMENT_GLOB 0x02
+#define XYZSH_ARGUMENT_OPTION 0x04
+#define XYZSH_ARGUMENT_QUOTED 0x08
 
 #define XYZSH_ARG_SIZE_MAX 65536*3
 #define XYZSH_REDIRECT_SIZE_MAX 72
@@ -276,6 +278,7 @@ typedef struct {
 #define COMPLETION_FLAGS_AFTER_REDIRECT 0x20000
 #define COMPLETION_FLAGS_AFTER_EQUAL 0x40000
 #define COMPLETION_FLAGS_HERE_DOCUMENT 0x80000
+#define COMPLETION_FLAGS_AFTER_SPACE 0x100000
 
 struct block_obj
 {
@@ -371,7 +374,7 @@ typedef struct {
 #define IS_MARKED(o) ((o)->mFlags & GC_MARK)
 #define SET_MARK(o) ((o)->mFlags |= GC_MARK)
 
-#define TYPE(o) ((o)->mFlags & 0xFF)
+#define STYPE(o) ((o)->mFlags & 0xFF)
 #define CLEAR_MARK(o) ((o)->mFlags &= ~GC_MARK)
 
 #define T_STRING 1
@@ -462,13 +465,14 @@ sObject* extobj_new_on_gc(void* object, fExtObjMarkFun mark_fun, fExtObjFreeFun
 void uobject_delete_on_gc(sObject* self);
 void uobject_delete_on_stack(sObject* self);
 int uobject_gc_children_mark(sObject* self);
+BOOL uobject_erase(sObject* self, char* key);
 
 sObject* uobject_new_on_gc(int size, sObject* parent, char* name, BOOL user_object);
 sObject* uobject_new_on_stack(int size, sObject* object, char* name);
 void uobject_put(sObject* self, char* key, void* item);
 void uobject_init(sObject* self);
 void uobject_root_init(sObject* self);
-void readline_object_init(sObject* self);
+void editline_object_init(sObject* self);
 void curses_object_init(sObject* self);
 void uobject_put(sObject* self, char* key, void* item);
 void* uobject_item(sObject* self, char* key);
@@ -518,6 +522,7 @@ sObject* fd_new_on_stack();
 void fd_delete_on_stack(sObject* self);
 #define FD_NEW_STACK() fd_new_on_stack()
 BOOL fd_write(sObject* self, char* str, int size);
+BOOL fd_writec(sObject* self, char c);
 
 void fd_split(sObject* self, eLineField lf, BOOL pomch_to_last_line, BOOL chomp_before_split, BOOL split_with_all_item_chompped);
 BOOL fd_guess_lf(sObject* self, eLineField* lf);
@@ -588,6 +593,7 @@ BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_mrun(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_errmsg(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_prompt(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_rprompt(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_try(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_rehash(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
@@ -648,7 +654,7 @@ BOOL cmd_raise(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_cd(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_popd(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_pushd(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
-BOOL cmd_block(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_ablock(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_index(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_rindex(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_uc(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
@@ -730,8 +736,8 @@ BOOL forground_job(int num);
 BOOL background_job(int num);
 void xyzsh_wait_background_job();
 
-void xyzsh_readline_init(BOOL runtime_script);
-void xyzsh_readline_final();
+void xyzsh_editline_init();
+void xyzsh_editline_final();
 
 extern sObject* gJobs;
 
@@ -762,10 +768,11 @@ extern sObject* gCurrentObject;
 extern sObject* gStackFrames;
 extern sObject* gMemChecker;
 extern sObject* gGlobalPipe;
-extern sObject* gInheritObject;
+extern sObject* gInheritMethod;
 extern sRunInfo* gRunInfoOfRunningObject;
 
 BOOL bufsiz_write(int fd, char* buf, int buf_len);
+BOOL bufsiz_read(int fd, ALLOC char** result, int* result_len, int* result_size);
 
 BOOL load_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo, char** argv, int argc);
 BOOL load_so_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo);
@@ -776,8 +783,6 @@ BOOL sCommand_option_item(sCommand* self, char* key);
 char* sCommand_option_with_argument_item(sCommand* self, char* key);
 BOOL sCommand_put_option(sCommand* self, MANAGED char* key);
 BOOL sCommand_put_option_with_argument(sCommand* self, MANAGED char* key, MANAGED char* argument);
-BOOL sCommand_expand_env(sCommand* command, sObject* object, sObject* nextin, sObject* nextout, sRunInfo* runinfo);
-BOOL sCommand_expand_env_redirect(sCommand* command, sObject* nextin, sRunInfo* runinfo);
 void sCommand_sweep_runtime_info(sCommand* command);
 void sStatment_sweep_runtime_info(sStatment* self);
 
@@ -785,12 +790,10 @@ BOOL run_function(sObject* fun, sObject* nextin, sObject* nextout, sRunInfo* run
 BOOL run_completion(sObject* compl, sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 
 #define PARSER_MAGIC_NUMBER_ENV 3
-#define PARSER_MAGIC_NUMBER_OPTION 5
 
 #include <oniguruma.h>
 int get_onig_regex(regex_t** reg, sRunInfo* runinfo, char* regex);
 
-int readline_signal();
 void clear_matching_info_variable();
 #include <stdarg.h>
 void xyzsh_read_rc_core(char* path);
@@ -801,14 +804,9 @@ void run_init(enum eAppType app_type);
 void run_final();
 
 extern sObject* gPrompt;
+extern sObject* gRPrompt;
 extern sObject* gDirStack;
 
-void readline_write_history();
-void readline_read_history();
-void readline_no_completion();
-void readline_completion();
-void readline_completion_user_castamized(sObject* block);
-
 char* xstrncpy(char* src, char* des, int size);
 char* xstrncat(char* src, char* des, int size);
 
@@ -821,11 +819,11 @@ BOOL get_object_prefix_and_name_from_argument(sObject** object, sObject* name, c
 //////////////////////////////////////////////////////////////////////
 BOOL xyzsh_load_file(char* fname, char** argv, int argc, sObject* current_object);
 void xyzsh_opt_c(char* cmd, char** argv, int argc);
-void xyzsh_readline_interface(char* cmdline, int cursor_point, char** argv, int argc, BOOL exit_in_spite_ofjob_exist);
+void xyzsh_editline_interface(char* cmdline, int cursor_point, char** argv, int argc, BOOL exit_in_spite_ofjob_exist, BOOL welcome_msg);
 
 void xyzsh_add_inner_command(sObject* object, char* name, fXyzshNativeFun command, int arg_num, ...);
 
-BOOL xyzsh_readline_interface_onetime(int* rcode, char* cmdline, int cursor_point, char* source_name, char** argv, int argc, fXyzshJobDone xyzsh_job_done_);
+BOOL xyzsh_editline_interface_onetime(int* rcode, char* cmdline, int cursor_point, char* source_name, char** argv, int argc, fXyzshJobDone xyzsh_job_done_);
 char* xyzsh_job_title(int n);
 int xyzsh_job_num();
 BOOL xyzsh_run(int* rcode, sObject* block, char* source_name, fXyzshJobDone xyzsh_job_done_, sObject* nextin, sObject* nextout, int argc, char** argv, sObject* current_object);
@@ -838,5 +836,22 @@ sObject* access_object(char* name, sObject** current, sObject* running_object);
 sObject* access_object2(char* name, sObject* current, sObject* running_object);
 sObject* access_object3(char* name, sObject** current);
 
+ALLOC char* editline(char* prompt, char* rprompt, char* text, int cursor_pos);
+ALLOC char* editline_user_castamized(char* prompt, char* rprompt, char* text, int cursor_pos, sObject* block);
+ALLOC char* editline_no_completion(char* prompt, char* rprompt, char* text, int cursor_pos);
+
+BOOL nfun_option_with_argument(sObject* self, char* key);
+BOOL class_option_with_argument(sObject* self, char* key);
+BOOL fun_option_with_argument(sObject* self, char* key);
+void fd_clear(sObject* self);
+BOOL fd_flash(sObject* self, int fd);
+BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL add_object_to_objects_in_pipe(sObject* object, sRunInfo* runinfo, sCommand* command);
+BOOL xyzsh_readline_interface_onetime(int* rcode, char* cmdline, int cursor_pos, char* source_name, char** argv, int argc, fXyzshJobDone xyzsh_job_done_);
+void xyzsh_readline_interface(char* cmdline, int cursor_point, char** argv, int argc, BOOL exit_in_spite_ofjob_exist, BOOL welcome_msg);
+
+void load_init();
+void load_final();
+
 #endif
 
